// Copyright (c)2002 Eric Carlson
// This source is licensed under the BSD license.
// Please see the LICENSE.txt file included with the distribution for more information.

#import <Cocoa/Cocoa.h>

#import "USBMonitor.h"
#import "RezChangeNotification.h"

@interface SliderController : NSObject
{
    //Screen tab
    IBOutlet id brightnessSlider;
    IBOutlet id contrastSlider;
    IBOutlet id horizontalPositionSlider;
    IBOutlet id horizontalSizeSlider;
    IBOutlet id verticalPositionSlider;
    IBOutlet id verticalSizeSlider;
    IBOutlet id brightnessValue;
    IBOutlet id contrastValue;
    IBOutlet id horizontalPositionValue;
    IBOutlet id horizontalSizeValue;
    IBOutlet id verticalPositionValue;
    IBOutlet id verticalSizeValue;

    //geometry tab
    IBOutlet id geometrySlider;
    IBOutlet id geometrySelector;
    IBOutlet id geometryValue;

    //advanced tab
    IBOutlet id horizontalMisconvergenceSlider;
    IBOutlet id horizontalMisconvergenceValue;
    IBOutlet id verticalMisconvergenceSlider;
    IBOutlet id verticalMisconvergenceValue;

    IBOutlet NSWindow *revertProgressSheet;
    IBOutlet NSWindow *mainWindow;

    IBOutlet id revertProgress;
    
    USBMonitor *myMonitor; //the object representing the monitor
    long *originalSettings; //the settings array grabbed at startup
    OurDMNotifyData data;
}

//advanced tab
- (IBAction)changeHorizontalMisconvergence:(id)sender;
- (IBAction)changeVerticalMisconvergence:(id)sender;
- (IBAction)degauss:(id)sender;

//geometry tab
- (IBAction)updateGeometrySliderTarget:(id)sender;
- (IBAction)changeGeometry:(id)sender;

//screen tab
- (IBAction)changeBrightness:(id)sender;
- (IBAction)changeContrast:(id)sender;
- (IBAction)changeHorizontalPosition:(id)sender;
- (IBAction)changeHorizontalSize:(id)sender;
- (IBAction)changeVerticalPosition:(id)sender;
- (IBAction)changeVerticalSize:(id)sender;

//helper
- (void)updateDisplayedSettings;

//callback selector for no monitor error sheet
- (void)noMonitorSheetDidEnd:(NSWindow *)sheet
                  returnCode:(int)returnCode
                 contextInfo:(void *)contextInfo;

//methods for dealing with revert sheets
- (void)updateRevertProgress;
- (void)raiseRevertProgressSheet;
- (void)endRevertProgressSheet;
- (void)revertProgressSheetDidEnd:(NSWindow *)sheet
                       returnCode:(int)returnCode
                      contextInfo:(void *)contextInfo;
- (IBAction)revertSettings:(id)sender;
- (void)revertSheetDidDismiss:(NSWindow *)sheet
                   returnCode:(int)returnCode
                  contextInfo:(void*)contextInfo;
@end
