// Copyright (c)2002 Eric Carlson
// This source is licensed under the BSD license.
// Please see the LICENSE.txt file included with the distribution for more information.

#import "SliderController.h"

@implementation SliderController

//from Monitor->Revert Settings... menu item
- (IBAction)revertSettings:(id)sender
{
    NSBeginAlertSheet(@"Revert Settings", @"No", @"Yes", nil, mainWindow, self, NULL, @selector(revertSheetDidDismiss:returnCode:contextInfo:), nil, @"Are you sure you want to revert this monitor's settings for this mode?");
}

//called when user dismisses revert sheet
- (void)revertSheetDidDismiss:(NSWindow *)sheet
                   returnCode:(int)returnCode
                  contextInfo:(void *)contextInfo
{
    if(returnCode == NSAlertAlternateReturn) { //Yes was pressed
        [self raiseRevertProgressSheet];
        [myMonitor updateSettingsUsingArray:originalSettings
                       withProgressSelector:@selector(updateRevertProgress)
                           selectorWhenDone:@selector(endRevertProgressSheet)
                                    ofClass:self];
    }
}

//callback to update progress bar as revert is occurring
- (void)updateRevertProgress
{
    [revertProgress incrementBy:1.00];
    [revertProgress displayIfNeeded]; //make sure it's updated
}

//put up the revert progress sheet
- (void)raiseRevertProgressSheet
{
    [NSApp beginSheet:revertProgressSheet
       modalForWindow:mainWindow
        modalDelegate:self
       didEndSelector:@selector(revertProgressSheetDidEnd:returnCode:contextInfo:)
          contextInfo:nil];    
}

//callback passed to revert procedure to be called when reverting settings is finished
- (IBAction)endRevertProgressSheet
{
    //hide sheet
    [revertProgressSheet orderOut:self];

    //zero progress indicator
    [revertProgress setDoubleValue:0];

    //return to normal event handling
    [NSApp endSheet:revertProgressSheet returnCode:1];    
}

//called when revert progress sheet is done
- (void)revertProgressSheetDidEnd:(NSWindow *)sheet
                       returnCode:(int)returnCode
                      contextInfo:(void *)contextInfo
{
    [self updateDisplayedSettings];
}

//terminate the application when the user dismisses the no monitor alert sheet
- (void)noMonitorSheetDidEnd:(NSWindow *)sheet
                  returnCode:(int)returnCode
                 contextInfo:(void *)contextInfo
{
    [NSApp terminate:nil];
}

//called when the user changes a geometry radio button, or
//to update the slider based on the currently selected radio button
- (IBAction)updateGeometrySliderTarget:(id)sender
{
    int selectedValue = [[sender selectedCell] tag];
    switch(selectedValue) {
        case 0: //horiz pincushion
            [geometrySlider setFloatValue:[myMonitor horizontalPincushion]];
            [geometryValue setFloatValue:([myMonitor horizontalPincushion]/255.0)*100];
            break;
        case 1: //horiz pincushion balance
            [geometrySlider setFloatValue:[myMonitor horizontalPincushionBalance]];
            [geometryValue setFloatValue:([myMonitor horizontalPincushionBalance]/255.0)*100];
            break;
        case 2: //keystone balance
            [geometrySlider setFloatValue:[myMonitor keystoneBalance]];
            [geometryValue setFloatValue:([myMonitor keystoneBalance]/255.0)*100];
            break;
        case 3: //keystone
            [geometrySlider setFloatValue:[myMonitor keystone]];
            [geometryValue setFloatValue:([myMonitor keystone]/255.0)*100];
            break;
        case 4: //rotation
            [geometrySlider setFloatValue:[myMonitor rotation]];
            [geometryValue setFloatValue:([myMonitor rotation]/255.0)*100];
            break;
        case 5: //top corner distortion control
            [geometrySlider setFloatValue:[myMonitor topCornerDistortionControl]];
            [geometryValue setFloatValue:([myMonitor topCornerDistortionControl]/255.0)*100];
            break;
        case 6: //top corner distortion balance
            [geometrySlider setFloatValue:[myMonitor topCornerDistortionBalance]];
            [geometryValue setFloatValue:([myMonitor topCornerDistortionBalance]/255.0)*100];
            break;
        case 7: //bottom corner distortion control
            [geometrySlider setFloatValue:[myMonitor bottomCornerDistortionControl]];
            [geometryValue setFloatValue:([myMonitor bottomCornerDistortionControl]/255.0)*100];
            break;
        default:
            NSLog(@"unknown tag returned\n");
            break;
    }
}

//called when the user moves the geometry slider
- (IBAction)changeGeometry:(id)sender
{
    int selectedControl = [[geometrySelector selectedCell] tag];
    float myValue = [sender floatValue];
    [geometryValue setFloatValue:(myValue/255.0)*100];
    switch(selectedControl) {
        case 0: //horiz pincushion
            [myMonitor setHorizontalPincushion:myValue];
            break;
        case 1: //horiz pincushion balance
            [myMonitor setHorizontalPincushionBalance:myValue];
            break;
        case 2: //keystone balance
            [myMonitor setKeystoneBalance:myValue];
            break;
        case 3: //keystone
            [myMonitor setKeystone:myValue];
            break;
        case 4: //rotation
            [myMonitor setRotation:myValue];
            break;
        case 5: //top corner distortion control
            [myMonitor setTopCornerDistortionControl:myValue];
            break;
        case 6: //top corner distortion balance
            [myMonitor setTopCornerDistortionBalance:myValue];
            break;
        case 7: //bottom corner distortion control
            [myMonitor setBottomCornerDistortionControl:myValue];
            break;
        default:
            NSLog(@"unknown tag returned\n");
            break;
    }
}

//the following change* selectors are called when the user moves the corresponding slider
- (IBAction)changeVerticalMisconvergence:(id)sender
{
    float myValue = [sender floatValue];
    [verticalMisconvergenceValue setFloatValue:(myValue/255)*100];
    [myMonitor setVerticalMisconvergence:myValue];
}


- (IBAction)changeHorizontalMisconvergence:(id)sender
{
    float myValue = [sender floatValue];
    [horizontalMisconvergenceValue setFloatValue:(myValue/255)*100];
    [myMonitor setHorizontalMisconvergence:myValue];
}

- (IBAction)changeBrightness:(id)sender
{
    float myValue = [sender floatValue];
    [brightnessValue setFloatValue:(myValue/255)*100];
    [myMonitor setBrightness:myValue];
}

- (IBAction)changeContrast:(id)sender
{
    float myValue = [sender floatValue];
    [contrastValue setFloatValue:(myValue/255)*100];
    [myMonitor setContrast:myValue];
}

- (IBAction)changeHorizontalPosition:(id)sender
{
    float myValue = [sender floatValue];
    [horizontalPositionValue setFloatValue:(myValue/255)*100];
    [myMonitor setHorizontalPosition:myValue];
}

- (IBAction)changeHorizontalSize:(id)sender
{
    float myValue = [sender floatValue];
    [horizontalSizeValue setFloatValue:(myValue/255)*100];
    [myMonitor setHorizontalSize:myValue];
}

- (IBAction)changeVerticalPosition:(id)sender
{
    float myValue = [sender floatValue];
    [verticalPositionValue setFloatValue:(myValue/255)*100];
    [myMonitor setVerticalPosition:myValue];
}

- (IBAction)changeVerticalSize:(id)sender
{
    float myValue = [sender floatValue];
    [verticalSizeValue setFloatValue:(myValue/255)*100];
    [myMonitor setVerticalSize:myValue];
}

//called when degauss button is pressed
- (IBAction)degauss:(id)sender
{
    [myMonitor degauss];
}

//set up our state on launch
- (void)awakeFromNib
{
    myMonitor = [[USBMonitor alloc] init];
    if(myMonitor == nil) //suitable device not found
    {
        //window must be in the front to attach a sheet to it for some stupid reason (document THAT, Apple)
        [mainWindow makeKeyAndOrderFront:self]; 
        NSBeginCriticalAlertSheet(nil, @"Quit", nil, nil, mainWindow, self, @selector(noMonitorSheetDidEnd:returnCode:contextInfo:), NULL, nil, @"No suitable monitor was found.");
    }

    //get settings as they are before user mucks with them so they can revert if desired
    originalSettings = [myMonitor settingsArray];

    //set up the data struct that is passed to the notifcation callback
    //that's called by the (Carbon) Display Manager when the display mode changes.
    //this contains the information necessary to call an Objective-C selector in this instance
    //from a plain C function
    data.updateMethod = (void (*)(id, SEL))[self methodForSelector:@selector(updateAfterModeChange)];
    data.updateMethodSelf = (id) self;
    data.updateMethodSelector = @selector(updateDisplayedSettings);
    registerNotification(&data);

    //set up sliders with current values from monitor
    [self updateDisplayedSettings];
}

//callback to be called by the Display Manager notification callback if the display mode changes
- (void)updateAfterModeChange
{
    //free the old settings before replacing them
    free((void*)originalSettings);
    
    //grab the settings for this mode so we can revert to them
    originalSettings = [myMonitor settingsArray];

    //finally, update the GUI with the current settings
    [self updateDisplayedSettings];
}

//update the GUI sliders to the current values
- (void)updateDisplayedSettings
{
    CFDictionaryRef newMode = NULL;
    CFNumberRef number = NULL;
    long width,height,refresh;

    //get current display mode info
    if( (newMode = CGDisplayCurrentMode(kCGDirectMainDisplay)) != NULL) {
        number = CFDictionaryGetValue(newMode, kCGDisplayWidth);
        CFNumberGetValue( number, kCFNumberLongType, &width ) ;

        number =  CFDictionaryGetValue(newMode, kCGDisplayHeight);
        CFNumberGetValue( number, kCFNumberLongType, &height ) ;

        number = CFDictionaryGetValue(newMode, kCGDisplayRefreshRate);
        CFNumberGetValue( number, kCFNumberLongType, &refresh ) ;
    }

    //update the tile to reflect the new display mode info
    [mainWindow setTitle:[NSString stringWithFormat:@"Diamond Control X: %ld x %ld @ %ld hz",width,height,refresh]];
    
    //set sliders to current values stored in monitor
    [brightnessSlider setFloatValue:[myMonitor brightness]];
    [brightnessValue setFloatValue:([myMonitor brightness]/255.0)*100];
    [contrastSlider setFloatValue:[myMonitor contrast]];
    [contrastValue setFloatValue:([myMonitor contrast]/255.0)*100];
    [horizontalPositionSlider setFloatValue:[myMonitor horizontalPosition]];
    [horizontalPositionValue setFloatValue:([myMonitor horizontalPosition]/255.0)*100];
    [horizontalSizeSlider setFloatValue:[myMonitor horizontalSize]];
    [horizontalSizeValue setFloatValue:([myMonitor horizontalSize]/255.0)*100];
    [verticalPositionSlider setFloatValue:[myMonitor verticalPosition]];
    [verticalPositionValue setFloatValue:([myMonitor verticalPosition]/255.0)*100];
    [verticalSizeSlider setFloatValue:[myMonitor verticalSize]];
    [verticalSizeValue setFloatValue:([myMonitor verticalSize]/255.0)*100];

    [self updateGeometrySliderTarget:nil];

    [horizontalMisconvergenceSlider setFloatValue:[myMonitor horizontalMisconvergence]];
    [horizontalMisconvergenceValue setFloatValue:([myMonitor horizontalMisconvergence]/255.0)*100];
    [verticalMisconvergenceSlider setFloatValue:[myMonitor verticalMisconvergence]];
    [verticalMisconvergenceValue setFloatValue:([myMonitor verticalMisconvergence]/255.0)*100];    
}

//dispose of things... i'm not sure if/when this is ever called
- (void)dealloc
{
    NSLog(@"SliderController: dealloc called\n");
    free((void*)originalSettings);
    removeNotification(&data);
    [myMonitor dealloc];
    [super dealloc];
}

@end
