// Copyright (c)2002 Eric Carlson
// This source is licensed under the BSD license.
// Please see the LICENSE.txt file included with the distribution for more information.

#import <Foundation/Foundation.h>
#import "HID_Utilities_External.h"

//this is overkill since the USB Monitor Control Class Spec for VESA controls
//doesn't have usage values up to 0xFF, but just to be safe (it's a very sparse array to begin with)
#define MYELEMENTS_LENGTH 255

@interface USBMonitor : NSObject
{
    pRecElement myElements[MYELEMENTS_LENGTH]; //HID elements for myMonitor
    pRecDevice myMonitor; //the USB monitor
}

//set values
- (BOOL)degauss;
- (BOOL)setBrightness:(float)value;
- (BOOL)setContrast:(float)value;
- (BOOL)setHorizontalPosition:(float)value;
- (BOOL)setHorizontalSize:(float)value;
- (BOOL)setVerticalPosition:(float)value;
- (BOOL)setVerticalSize:(float)value;
- (BOOL)setHorizontalPincushion:(float)value;
- (BOOL)setHorizontalPincushionBalance:(float)value;
- (BOOL)setHorizontalMisconvergence:(float)value;
- (BOOL)setVerticalMisconvergence:(float)value;
- (BOOL)setKeystoneBalance:(float)value;
- (BOOL)setKeystone:(float)value;
- (BOOL)setRotation:(float)value;
- (BOOL)setTopCornerDistortionControl:(float)value;
- (BOOL)setTopCornerDistortionBalance:(float)value;
- (BOOL)setBottomCornerDistortionControl:(float)value;

//get current values
- (float)brightness;
- (float)contrast;
- (float)horizontalPosition;
- (float)horizontalSize;
- (float)verticalPosition;
- (float)verticalSize;
- (float)horizontalPincushion;
- (float)horizontalPincushionBalance;
- (float)horizontalMisconvergence;
- (float)verticalMisconvergence;
- (float)keystoneBalance;
- (float)keystone;
- (float)rotation;
- (float)topCornerDistortionControl;
- (float)topCornerDistortionBalance;
- (float)bottomCornerDistortionControl;

//methods to handle bulk actions
- (void)updateSettingsUsingArray:(long*)settings
            withProgressSelector:(SEL)updateProgress
                selectorWhenDone:(SEL)finish
                         ofClass:(NSObject *)theClass;
- (long*)settingsArray;

//private methods used by above methods
- (long)getHIDElementValue:(int)key;
- (BOOL)setHIDElement:(int)key withValue:(long)value;

@end
