// Copyright (c)2002 Eric Carlson
// This source is licensed under the BSD license.
// Please see the LICENSE.txt file included with the distribution for more information.

// Based on information from USB Monitor Control Class Specification, Revision 1.0 (January 5, 1998).
// Available from http://www.usb.org/developers/data/devclass/usbmon10.pdf

#ifndef _USBMonitorControlClassSpec_h_
#define _USBMonitorControlClassSpec_h_

// Identifiers for Mitsubishi Diamond Pro 900U
long kMitsuDiamondPro900U_ProdID = 81;
long kMitsuDiamondPro900U_VendID = 1106;

enum {
    kHIDPage_Monitor 	= 0x80,
    kHIDPage_Enum	= 0x81,
    kHIDPage_VESA	= 0x82
};

enum {
    kHIDMonitorUsagePage_Control = 0x01,
    kHIDMonitorUsagePage_EDID    = 0x02,
    kHIDMonitorUsagePage_VDIF    = 0x03,
    kHIDMonitorUsagePage_VESA    = 0x04
};

//element usages for page kHIDPage_VESA
enum {
    kHIDVESAUsagePage_Brightness	= 0x10, 				//900U
    kHIDVESAUsagePage_Contrast		= 0x12, 				//900U
    
    kHIDVESAUsagePage_Red		= 0x16,
    kHIDVESAUsagePage_Green		= 0x18,
    kHIDVESAUsagePage_Blue		= 0x1A,
    
    kHIDVESAUsagePage_Focus		= 0x1C,
    
    kHIDVESAUsagePage_HorizPosition	= 0x20, 				//900U
    kHIDVESAUsagePage_HorizSize		= 0x22, 				//900U
    kHIDVESAUsagePage_HorizPin		= 0x24,					//900U
    kHIDVESAUsagePage_HorizPinBal	= 0x26,					//900U
    kHIDVESAUsagePage_HorizMisConv	= 0x28,					//900U
    kHIDVESAUsagePage_HorizLin		= 0x2A,
    kHIDVESAUsagePage_HorizLinBal	= 0x2C,
    
    kHIDVESAUsagePage_VertPosition	= 0x30, 				//900U
    kHIDVESAUsagePage_VertSize		= 0x32,					//900U
    kHIDVESAUsagePage_VertPin		= 0x34,
    kHIDVESAUsagePage_VertPinBal	= 0x36,
    kHIDVESAUsagePage_VertMisConv	= 0x38,					//900U
    kHIDVESAUsagePage_VertLin		= 0x3A,
    kHIDVESAUsagePage_VertLinBal	= 0x3C,
    
    kHIDVESAUsagePage_KeyBalance	= 0x40, //Parallelogram Distortion	//900U
    kHIDVESAUsagePage_Key		= 0x42, //Trapezoidal Distortion	//900U
    kHIDVESAUsagePage_Tilt		= 0x44, //Rotation			//900U
    kHIDVESAUsagePage_TCDistControl 	= 0x46, //TC = Top Corner		//900U
    kHIDVESAUsagePage_TCDistBalance 	= 0x48, 				//900U
    kHIDVESAUsagePage_BCDistControl 	= 0x4A, //BC = Bottom Corner		//900U
    kHIDVESAUsagePage_BCDistBalance 	= 0x4C,
    
    kHIDVESAUsagePage_HorizMoire	= 0x56,
    kHIDVESAUsagePage_VertMoire		= 0x58,
    
    kHIDVESAUsagePage_RedBlackLvl 	= 0x6C,
    kHIDVESAUsagePage_GreenBlackLvl 	= 0x6E,
    kHIDVESAUsagePage_BlueBlackLvl	= 0x70,
    
    //Non-contiguous Read/Write Controls (ENUMs)
    kHIDVESAUsagePage_InputLevel	= 0x5E,
    kHIDVESAUsagePage_InputSource	= 0x60,
    kHIDVESAUsagePage_OSD		= 0xCA, //On Screen Display
    kHIDVESAUsagePage_StereoMode	= 0xD4,
    
    //Non-contiguous Read-Only Controls (ENUMs)
    kHIDVESAUsagePage_AutoSize		= 0xA2,
    kHIDVESAUsagePage_PolarityHoriz 	= 0xA4,
    kHIDVESAUsagePage_PolarityVert	= 0xA6,
    kHIDVESAUsagePage_SynchType		= 0xA8,
    kHIDVESAUsagePage_ScreenOrient	= 0xAA,
    kHIDVESAUsagePage_HorizFreq		= 0xAC,
    kHIDVESAUsagePage_VertFreq		= 0xAE,
    
    //Non-contiguous Write-only Controls
    kHIDVESAUsagePage_Degauss		= 0x01, //non-zero means start degauss	//900U
    kHIDVESAUsagePage_Settings		= 0xB0 //Save/Restore settings (ENUM)
};

#endif